@@ -11,10 +11,12 @@
 from typing import Optional, Dict
 import __future__
 import sys
-import numpy
+import os
 import threading
 import re
 
+import numpy
+
 is_cpu_amd_intel = False # DEPRECATION WARNING: WILL BE REMOVED IN FUTURE RELEASE
 from numexpr import interpreter, expressions, use_vml
 from numexpr.utils import CacheDict
@@ -261,17 +263,22 @@ class Immediate(Register):
         return 'Immediate(%d)' % (self.node.value,)
 
 
-_forbidden_re = re.compile('[\;[\:]|__|\.[abcdefghjklmnopqstuvwxyzA-Z_]')
-def stringToExpression(s, types, context):
+_flow_pat = r'[\;\[\:]'
+_dunder_pat = r'__[\w]+__'
+_attr_pat = r'\.\b(?!(real|imag|[eE]?[+-]?\d+)\b)'
+_blacklist_re = re.compile(f'{_flow_pat}|{_dunder_pat}|{_attr_pat}')
+
+def stringToExpression(s, types, context, sanitize: bool=True):
     """Given a string, convert it to a tree of ExpressionNode's.
     """
     # sanitize the string for obvious attack vectors that NumExpr cannot 
     # parse into its homebrew AST. This is to protect the call to `eval` below.
     # We forbid `;`, `:`. `[` and `__`, and attribute access via '.'.
     # We cannot ban `.real` or `.imag` however...
-    no_whitespace = re.sub(r'\s+', '', s)
-    if _forbidden_re.search(no_whitespace) is not None:
-        raise ValueError(f'Expression {s} has forbidden control characters.')
+    if sanitize:
+        no_whitespace = re.sub(r'\s+', '', s)
+        if _blacklist_re.search(no_whitespace) is not None:
+            raise ValueError(f'Expression {s} has forbidden control characters.')
     
     old_ctx = expressions._context.get_current_context()
     try:
@@ -558,7 +565,7 @@ def getContext(kwargs, _frame_depth=1):
     return context
 
 
-def precompile(ex, signature=(), context={}):
+def precompile(ex, signature=(), context={}, sanitize: bool=True):
     """
     Compile the expression to an intermediate form.
     """
@@ -566,7 +573,7 @@ def precompile(ex, signature=(), context={}):
     input_order = [name for (name, type_) in signature]
 
     if isinstance(ex, str):
-        ex = stringToExpression(ex, types, context)
+        ex = stringToExpression(ex, types, context, sanitize)
 
     # the AST is like the expression, but the node objects don't have
     # any odd interpretations
@@ -612,7 +619,7 @@ def precompile(ex, signature=(), context={}):
     return threeAddrProgram, signature, tempsig, constants, input_names
 
 
-def NumExpr(ex, signature=(), **kwargs):
+def NumExpr(ex,  signature=(), sanitize: bool=True, **kwargs):
     """
     Compile an expression built using E.<variable> variables to a function.
 
@@ -629,7 +636,7 @@ def NumExpr(ex, signature=(), **kwargs):
     # translated to either True or False).
     _frame_depth = 1
     context = getContext(kwargs, _frame_depth=_frame_depth)
-    threeAddrProgram, inputsig, tempsig, constants, input_names = precompile(ex, signature, context)
+    threeAddrProgram, inputsig, tempsig, constants, input_names = precompile(ex, signature, context, sanitize=sanitize)
     program = compileThreeAddrForm(threeAddrProgram)
     return interpreter.NumExpr(inputsig.encode('ascii'),
                                tempsig.encode('ascii'),
@@ -710,8 +717,8 @@ def getType(a):
     raise ValueError("unknown type %s" % a.dtype.name)
 
 
-def getExprNames(text, context):
-    ex = stringToExpression(text, {}, context)
+def getExprNames(text, context, sanitize: bool=True):
+    ex = stringToExpression(text, {}, context, sanitize)
     ast = expressionToAST(ex)
     input_order = getInputOrder(ast, None)
     #try to figure out if vml operations are used by expression
@@ -779,8 +786,9 @@ def validate(ex: str,
              order: str = 'K', 
              casting: str = 'safe', 
              _frame_depth: int = 2,
+             sanitize: Optional[bool] = None,
              **kwargs) -> Optional[Exception]:
-    """
+    r"""
     Validate a NumExpr expression with the given `local_dict` or `locals()`.
     Returns `None` on success and the Exception object if one occurs. Note that 
     you can proceed directly to call `re_evaluate()` if you use `validate()`
@@ -826,16 +834,23 @@ def validate(ex: str,
             like float64 to float32, are allowed.
           * 'unsafe' means any data conversions may be done.
 
+    sanitize: Optional[bool]
+        Both `validate` and by extension `evaluate` call `eval(ex)`, which is 
+        potentially dangerous on unsanitized inputs. As such, NumExpr by default 
+        performs simple sanitization, banning the character ':;[', the 
+        dunder '__[\w+]__', and attribute access to all but '.real' and '.imag'.
+        
+        Using `None` defaults to `True` unless the environment variable 
+        `NUMEXPR_SANITIZE=0` is set, in which case the default is `False`. 
+        Nominally this can be set via `os.environ` before `import numexpr`.
+
     _frame_depth: int
         The calling frame depth. Unless you are a NumExpr developer you should 
         not set this value.
 
     Note
     ----
-    Both `validate` and by extension `evaluate` call `eval(ex)`, which is 
-    potentially dangerous on unsanitized inputs. As such, NumExpr does some 
-    sanitization, banning the character ':;[', the dunder '__', and attribute
-    access to all but '.r' for real and '.i' for imag access to complex numbers.
+    
     """
     global _numexpr_last
 
@@ -843,12 +858,18 @@ def validate(ex: str,
         
         if not isinstance(ex, str):
             raise ValueError("must specify expression as a string")
+        
+        if sanitize is None:
+            if 'NUMEXPR_SANITIZE' in os.environ:
+                sanitize = bool(int(os.environ['NUMEXPR_SANITIZE']))
+            else:
+                sanitize = True
 
         # Get the names for this expression
         context = getContext(kwargs)
         expr_key = (ex, tuple(sorted(context.items())))
         if expr_key not in _names_cache:
-            _names_cache[expr_key] = getExprNames(ex, context)
+            _names_cache[expr_key] = getExprNames(ex, context, sanitize=sanitize)
         names, ex_uses_vml = _names_cache[expr_key]
         arguments = getArguments(names, local_dict, global_dict, _frame_depth=_frame_depth)
 
@@ -861,7 +882,7 @@ def validate(ex: str,
         try:
             compiled_ex = _numexpr_cache[numexpr_key]
         except KeyError:
-            compiled_ex = _numexpr_cache[numexpr_key] = NumExpr(ex, signature, **context)
+            compiled_ex = _numexpr_cache[numexpr_key] = NumExpr(ex, signature, sanitize=sanitize, **context)
         kwargs = {'out': out, 'order': order, 'casting': casting,
                 'ex_uses_vml': ex_uses_vml}
         _numexpr_last = dict(ex=compiled_ex, argnames=names, kwargs=kwargs)
@@ -875,9 +896,10 @@ def evaluate(ex: str,
              out: numpy.ndarray = None, 
              order: str = 'K', 
              casting: str = 'safe', 
+             sanitize: Optional[bool] = None,
              _frame_depth: int = 3,
              **kwargs) -> numpy.ndarray:
-    """
+    r"""
     Evaluate a simple array expression element-wise using the virtual machine.
 
     Parameters
@@ -920,6 +942,16 @@ def evaluate(ex: str,
             like float64 to float32, are allowed.
           * 'unsafe' means any data conversions may be done.
 
+    sanitize: bool
+        Both `validate` and by extension `evaluate` call `eval(ex)`, which is 
+        potentially dangerous on unsanitized inputs. As such, NumExpr by default 
+        performs simple sanitization, banning the character ':;[', the 
+        dunder '__[\w+]__', and attribute access to all but '.real' and '.imag'.
+
+        Using `None` defaults to `True` unless the environment variable 
+        `NUMEXPR_SANITIZE=0` is set, in which case the default is `False`. 
+        Nominally this can be set via `os.environ` before `import numexpr`.
+
     _frame_depth: int
         The calling frame depth. Unless you are a NumExpr developer you should 
         not set this value.
@@ -936,7 +968,7 @@ def evaluate(ex: str,
     # `getArguments`
     e = validate(ex, local_dict=local_dict, global_dict=global_dict, 
                  out=out, order=order, casting=casting, 
-                 _frame_depth=_frame_depth, **kwargs)
+                 _frame_depth=_frame_depth, sanitize=sanitize, **kwargs)
     if e is None:
         return re_evaluate(local_dict=local_dict, _frame_depth=_frame_depth)
     else:
